/*
 *  Copyright 2001-2010 Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This file is part of Graph Game.
 *
 *  Graph Game is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Graph Game is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Graph Game.  If not, see <http://www.gnu.org/licenses/>.
 */
package kryshen.graphg;

import java.util.Enumeration;
import java.util.Vector;

class Graph {
    Vector nodes = new Vector();
    Vector multiNodes = new Vector();
    Vector edges = new Vector();
    int width = 100;
    int height = 100;
    Node start = null;
    Node finish = null;

    public void removeEdge(Edge link) {
	if(edges.removeElement(link)) {
	    //if(link.node1.edges != null)
	    link.node1.edges.removeElement(link);
	    link.node1.multiNode.edges.removeElement(link);
	    //if(link.node2.edges != null)
	    link.node2.edges.removeElement(link);
	    link.node2.multiNode.edges.removeElement(link);
	}
    }

    public boolean addEdge(Edge link) {
	if(edges.contains(link))
	    return false;
	
 	edges.addElement(link);
	
	link.node1.edges.addElement(link);
	link.node1.multiNode.edges.addElement(link);
	link.node2.edges.addElement(link);

	if(!link.node1.multiNode.equals(link.node2.multiNode))
	    link.node2.multiNode.edges.addElement(link);
	
	return true;
    }

    public void addNode(Node node) {
	if(nodes.contains(node))
	    nodes.removeElement(node);
	/*if(!this.equals(node.graph)) {
	    node.graph = this;
	    if(node.getEdges().size() > 0)
		node.edges = null;
	}*/
	nodes.addElement(node);
	if(!multiNodes.contains(node.multiNode))
	    multiNodes.addElement(node.multiNode);
    }

    // wave algorithm
    public Path shortestPath(MultiNode v1, MultiNode v2) {
	Enumeration ei, ej;

	for (ei = multiNodes.elements(); ei.hasMoreElements(); ) {
	    ((MultiNode)ei.nextElement()).mark = Integer.MAX_VALUE;
	}

	Vector oldFront = new Vector(multiNodes.size());
	Vector newFront = new Vector(multiNodes.size());
	
	v1.mark = 0;
	oldFront.addElement(v1);

	do {
	    for (ei = oldFront.elements(); ei.hasMoreElements(); ) {
		MultiNode current = (MultiNode)ei.nextElement();

		for (ej = current.edges.elements(); ej.hasMoreElements(); ) {
		    Edge edge = (Edge)ej.nextElement();
		    
		    if (edge.enabled) {
			MultiNode next = edge.getAnotherNode(current);
			
			if (next.mark > current.mark + 1) {
			    next.mark = current.mark + 1;
			    next.pathEdge = edge;

			    if (next == v2) {				
				Path path = new Path(next.mark);
				while (next != v1) {
				    path.addElement(next.pathEdge);
				    next = next.pathEdge.getAnotherNode(next);
				}				
				return path;
			    }

			    newFront.addElement(next);
			}
		    }
		}
	    }

	    Vector t = oldFront;
	    oldFront = newFront;
	    newFront = t;
	    newFront.removeAllElements();

	} while (oldFront.size() > 0);

	return null;
    }
}
